#!/usr/bin/ksh
#   begin_generated_IBM_prolog                                          
#                                                                       
#   prolog.pro.samp.telcosp                                                 
#   ******************************************************************  
#   IBM Sample Source Materials                                         
#                                                                       
#   Sample source materials are supplied As-Is.                         
#   No warranty is expressed or implied.                                
#                                                                       
#   Product(s): 5724-O05 soa                                            
#                                                                       
#   Copyright IBM Corp. 2000, 2007                                      
#                                                                       
#   The source code for this program supplied under the terms of the    
#   End User License Agreement (EULA) that accompanied this product.    
#   ******************************************************************  
#                                                                       
#   end_generated_IBM_prolog                                            

#------------------------------------------------------------------
# Create the IBM DB2 database tables
# Version: 1.7.5.10
#
# Set/gather default information, then prompt user to verify
# or override values.
#
# To debug this script change the first line to
# "#!/usr/bin/ksh -x" or "#!/usr/bin/ksh -v".
# You may also want to comment out any "clear" commands in the
# script.
#------------------------------------------------------------------

#------------------------------------------------------------------
# The DB2 instance id and connection port might be in /etc/services.
# Example:
#   db2cdb2inst1    50000/tcp   # Connection port for DB2 instance db2inst1
#------------------------------------------------------------------

#------------------------------------------------------------------
# Display information to user
#------------------------------------------------------------------

function displayGUI {

  #clear
  print "Usage: crtsrvDb2.sh dbServer dbPort dbName dbAlias dbLocale"
  print "                    dbInstance dbInstancePW dbUser dbUserPW"
  print "                    ddlFile dbCreate dbLocal dbNodeName menuflag"
  print "These values will be used to create the database."
  print "Each value must be filled in."
  print "The menuflag should be set to TRUE to display the menu."
  print ""
  print "Press Enter to accept these values and continue, otherwise"
  print "  enter the number of the parameter you wish to override."
  print ""
  print "   1. Database server hostname (dbServer)......: " $dbServer
  print "      Recommended Value: fully qualified host name."
  print "   2. Database server connection port (dbPort).: " $dbPort
  print "   3. Database name (dbName)...................: " $dbName
  print "      Recommended Value: XDMSURDB"
  print "   4. Database alias (dbAlias).................: " $dbAlias
  print "      Recommended Value: XDMSUR"
  print "   5. Database locale (dbLocale)...............: " $dbLocale
  print "      Recommended Value: US"
  print "   6. Database server instance id (dbInstance).: " $dbInstance
  print "   7. Database server instance password (dbInstancePW).: " $dbInstancePW
  print "   8. Database userid (dbUser).................: " $dbUser
  print "   9. Database userid password (dbUserPW)......: " $dbUserPW
  print "  10. Path and file name of DDL file (ddlFile).: " $ddlFile
  print "  11. Database (re)create (dbCreate)...........: " $dbCreate
  print "      Recommended Value: TRUE"
  print "  12. Local Database (dbLocal) ................: " $dbLocal
  print "      Recommended Value: TRUE"              
  print "  13. Remote Database Server Node Name (dbNodeName)...: " $dbNodeName
  print "      Recommended Value: RDBSRV"
  print ""

  read input
  # [[ $input = "" ]] && rc="0" || rc=$input

  # rc must be an integer value.  Correct it if a typo was made.
  case $input in
    1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12 | 13) rc=$input ;;   # numeric input
    ""               ) rc="0"    ;;   # null input
    *                ) rc=88     ;;   # string input which cannot be passed by return command
  esac
  return $rc

} # end of function displayGUI

#------------------------------------------------------------------
# Program main code
#------------------------------------------------------------------

# process menu
dbServer=$1
dbPort=$2
dbName=$3
dbAlias=$4
dbLocale=$5
dbInstance=$6
dbInstancePW=$7
dbUser=$8
dbUserPW=$9
ddlFile=${10}
dbCreate=${11}
dbLocal=${12}
dbNodeName=${13}
menuflag=${14}

# handle menuflag not specified
if [[ -z $menuflag ]]
   then
      menuflag="TRUE"
fi 

if [[ -z $dbLocal ]]
   then
      dbLocal="TRUE"
fi

if [[ -z $dbServer ]]
   then
      dbServer="localhost"
fi

if [[ -z $dbNodeName ]]
   then
      dbNodeName="RDBSRV"
fi

if [[ -z $dbPort ]]
   then
      dbPort="50000"
fi

if [[ -z $dbName ]]
   then 
      dbName="XDMSURDB"
fi

if [[ -z $dbAlias ]]
   then 
      dbAlias="XDMSUR"
fi

if [[ -z $dbLocale ]]
   then
      dbLocale="US"
fi

if [[ -z $dbCreate ]]
   then
      dbCreate="TRUE"
fi

if [[ "$menuflag" = "TRUE" || "$menuflag" = "true" ]]
   then

   ((rc=99))
   while ((rc != 0))
     do
       [[ -e "$ddlFile" ]] || ((rc=10))
       case $rc in
         1) print "    Enter the database server hostname:"
            read input
            [[ -n $input ]] && dbServer=$input
            ;;

         2) print "    Enter the database server connection port:"
            read input
            [[ -n $input ]] && dbPort=$input
            ;;

         3) print "    Enter the database name:"
            read input
            [[ -n $input ]] && dbName=$input
            ;;

         4) print "    Enter the database alias:"
            read input
            [[ -n $input ]] && dbAlias=$input
            ;;

         5) print "    This is the territory code used by the database to control sort order"
            print "      and other reginal settings.  Examples are US, JP, CN, TW, Lat, etc."
            print "    Enter the database territory code:"
            read input
            [[ -n $input ]] && dbLocale=$input
            ;;

         6) print "    Enter the database server instance id:"
            read input
            [[ -n $input ]] && dbInstance=$input
            ;;

         7) print "    Enter the database server instance password:"
            read input
            [[ -n $input ]] && dbInstancePW=$input
            ;;

         8) print "    Enter the database server userid:"
            read input
            [[ -n $input ]] && dbUser=$input
            ;;

         9) print "    Enter the database server user password:"
            read input
            [[ -n $input ]] && dbUserPW=$input
            ;;

        10) print "    Enter the path and file name of the DDL file:"
            print "    This value must be correct before any other values can be changed."
            print "    Recommended value is the DDL file provided with the component"
            read input
            [[ -n $input ]] && ddlFile=$input
            ;;
        11) print "    Enter the boolean indicating whether to (re)create the database (TRUE or FALSE):"
            read input
            [[ -n $input ]] && dbCreate=$input
            ;;
        12) print "    Enter the boolean indicating whether the database is local (TRUE or FALSE):"
            read input
            [[ -n $input ]] && dbLocal=$input
            ;;
        13) print "    Enter the database server node name to use in the catalog:"
            read input
            [[ -n $input ]] && dbNodeName=$input
            ;;
                                                                                                                                                                    
         *)
           ;;
       esac

       if [[ -s $ddlFile ]]
       then
         displayGUI
       else    # request location of the ddl again
         print "    ERROR: Cannot find file $ddlFile"
         print "      ensure you include the path information to the DDL file"
         ((rc=10))
       fi

     done  # end of while loop
fi     # end of menuing block

if [[ -r "$ddlFile" ]]
  then
     if [[ "TRUE" = "$dbCreate" || "true" = "$dbCreate" ]]
       then
          print "Creating server database."

          # This command may be necessary if the database is in use.
          #db2 force applications all

          db2 uncatalog database $dbAlias

          # This command may be necessary if the database already exists in either
          # the local database directory or system database directory.
          # db2 catalog database $dbName

          # Execute the then block if the DB2 server is local.
          # Execute the else block if the DB2 server is remote, as in a cluster.
          if [[ "TRUE" = "$dbLocal" || "true" = "$dbLocal" ]]
            then
              db2 drop database $dbName
              db2 uncatalog node local
              db2 create database $dbName using codeset UTF-8 territory $dbLocale with \"WebSphere XDMS Data\"
              db2 catalog local node local 
              db2 catalog database $dbName as $dbAlias at node local
            else
              db2 uncatalog node $dbNodeName
              db2 catalog tcpip node $dbNodeName remote $dbServer server $dbPort
              db2 catalog database $dbName as $dbAlias at node $dbNodeName with \"WebSphere XDMS Data\"
          fi

          db2 connect to $dbAlias user $dbInstance using $dbInstancePW
          if [[ "$dbInstance" != "$dbUser" ]]
             then
                db2 grant DBADM on database to user $dbUser
          fi
         # db2 connect reset
         # db2 terminate
          
          ########################################################################
          #
          #   All columns (except CLOBs and BLOBs) must fit within the page size
          #   (default is 4K).  So if you need to increase column sizes, you may
          #   have to place the table in a tablespace with a larger page size.
          #   See the DB2 Administrator Guide for a more detailed explaination of
          #   the design rules and the DB2 SQL Reference for details of the
          #   command syntax.
          #
          #   If the SRVPOOL OR SRVSPACE already exist you will get errors and that is ok.
          #
          #   ** create bufferpool using 8K.  You can also use 16K or 32K.
         # db2 connect to $dbAlias user $dbInstance using $dbInstancePW
          db2 CREATE BUFFERPOOL SRVPOOL SIZE 250 PAGESIZE 8K
         # db2 connect reset
         # db2 terminate
          #
          #   Terminate all access to the database before the SRVPOOL will be active.
          #   !Note! this appears no longer to be necessary, but maybe on older versions of DB2
          #
          #   *****  Important  ****  Important  ****  Important  ****  Important **
          #   ** The chosen path must be writable by the database end user's ID.
          #   ** The directory must be empty or have the same database in it.
          #
         # db2 connect to $dbAlias user $dbInstance using $dbInstancePW
          line="db2 CREATE REGULAR TABLESPACE SRVSPACE PAGESIZE 8 K  MANAGED BY SYSTEM USING ('soatmp') EXTENTSIZE 32 OVERHEAD 24.1 PREFETCHSIZE 16 TRANSFERRATE 0.9 BUFFERPOOL SRVPOOL"
          $line
          db2 connect reset
          db2 terminate
          #
          ########################################################################
     else
          # not creating but ensure the user has DBADM and TABLESPACE and BUFFERPOOL are created
	  db2 connect to $dbAlias user $dbInstance using $dbInstancePW
          if [[ "$dbInstance" != "$dbUser" ]]
             then
                db2 grant DBADM on database to user $dbUser
          fi
         # db2 connect reset
         # db2 terminate
          
          ########################################################################
          #
          #   All columns (except CLOBs and BLOBs) must fit within the page size
          #   (default is 4K).  So if you need to increase column sizes, you may
          #   have to place the table in a tablespace with a larger page size.
          #   See the DB2 Administrator Guide for a more detailed explaination of
          #   the design rules and the DB2 SQL Reference for details of the
          #   command syntax.
          #
          #   If the SRVPOOL OR SRVSPACE already exist you will get errors and that is ok.
          #
          #   ** create bufferpool using 8K.  You can also use 16K or 32K.
         # db2 connect to $dbAlias user $dbInstance using $dbInstancePW
          db2 CREATE BUFFERPOOL SRVPOOL SIZE 250 PAGESIZE 8K
         # db2 connect reset
         # db2 terminate
          #
          #   Terminate all access to the database before the SRVPOOL will be active.
          #   !Note! this appears no longer to be necessary, but maybe on older versions of DB2
          #
          #   *****  Important  ****  Important  ****  Important  ****  Important **
          #   ** The chosen path must be writable by the database end user's ID.
          #   ** The directory must be empty or have the same database in it.
          #
         # db2 connect to $dbAlias user $dbInstance using $dbInstancePW
          line="db2 CREATE REGULAR TABLESPACE SRVSPACE PAGESIZE 8 K  MANAGED BY SYSTEM USING ('soatmp') EXTENTSIZE 32 OVERHEAD 24.1 PREFETCHSIZE 16 TRANSFERRATE 0.9 BUFFERPOOL SRVPOOL"
          $line
          db2 connect reset
          db2 terminate
          #
          ########################################################################
     fi

    # Now create the database tables. See the comment above regarding row size.
    db2 connect to $dbAlias user $dbUser using $dbUserPW

    # Tell DB2 to run the users DDL commands
    db2 -tvf $ddlFile

    db2 connect reset
    db2 terminate

  else
    print "ERROR: $ddlFile cannot be found or is not readable by this process.";
    exit
fi

print ""
print "= Note: ====================================================================="
print "  User "$dbUser" may have been granted DBADM authorities to database "$dbAlias"."
print ""
print "  You may want to revoke several of the authorities that have been granted."
print "  The user will require the authority necessary to access and update "$dbAlias"."
print ""
print "  To remove DBADM authority, issue the following commands:"
print "     db2 connect to "$dbAlias" user "$dbInstance
print "     db2 revoke DBADM,LOAD,IMPLICIT_SCHEMA on database from "$dbUser
print "     db2 revoke BINDADD,CREATE_NOT_FENCED on database from "$dbUser
print "     db2 terminate"
print ""
print ""
print "  $0 is complete."

exit
