#!/bin/sh
#   begin_generated_IBM_prolog                                          
#                                                                       
#   prolog.pro.lic.telcosp                                              
#   ******************************************************************  
#   IBM Licensed Source                                                 
#                                                                       
#   Product(s): 5724-O05                                                
#                                                                       
#   Copyright IBM Corp. 2007, 2007                                      
#                                                                       
#   The source code for this program is restricted to the terms         
#   specified by the End User License Agreement (EULA) that             
#   accompanied this product.                                           
#   ******************************************************************  
#                                                                       
#   end_generated_IBM_prolog                                            

# Usage function
usage ( ) {
        echo "Usage:"
        echo "    $0 <db_name> <schema_file> <db_user> <db_password>"
        echo ""
        echo "Example:"
        echo "    $0 xdmssibus cluster1schemas.lst db2inst1 dbpassword"
        echo ""
        echo "Note: <schema_file> is the filename of a file that was "
        echo " generated during install of the cluster.  The file   "
        echo " should contain the list of schema used by the cluster."
}

# Check the variables
if [ $# != 4 ]
then
        usage
        exit -1
fi

# Variables
db_name=$1
schema_file=$2
db_user=$3
db_password=$4

# Verify schema file exists
if [ ! -f ${schema_file} ]
then
	echo "schema file does not exist"
	exit 2
fi

# temporary file
tmp_file="/tmp/deleteXdmsSIBus.$$"
if [ `uname | cut -c -6 -` == "CYGWIN" ]
then
	tmp_file="$(cygpath -w "${tmp_file}")"
fi

# connect to the database 
db2 "connect to ${db_name} user ${db_user} using ${db_password}"

if [ ! $? -eq 0 ]
then
   echo "Error connecting to database"
   exit 1
fi

# read schema file line by line
cat ${schema_file} | \
while read schema_name
do
	db2 +o list tables for schema ${schema_name}
	if [ $? -ne 0 ]
	then
		echo "Schema ${schema_name} does not own any tables.  Skipping"
	else
		# Put drop commands in temporary file
		db2 +o -x -z ${tmp_file} "select 'DROP TABLE ' || RTRIM(TABSCHEMA) || '.' || RTRIM(tabname) || ';' from syscat.tables where tabschema = '${schema_name}' order by type desc"
		# append command to drop the schema
		echo "DROP SCHEMA ${schema_name} RESTRICT;" >> ${tmp_file}
	fi
done

# Confirm actions
echo "*** About to perform the following: "
cat ${tmp_file}
echo "*** OK to continue? (y/n)"
read ans
if [ "${ans}" != "y" ] && [ "${ans}" != "Y" ]
then
   echo "Aborting"
   rm -f ${tmp_file}
   exit 3
fi

# run the file
db2 -tvf ${tmp_file}


# Cleanup the temporary db2 file 
rm -f ${tmp_file}
