#
# XDMS WSAdmin Jython script Install Script
#
# This script creates the service integration bus and the JMS artifacts used by the 
# IMS Service Platform XDMS component.
#
# Usage:
# wsadmin.sh -lang jython -f <script name> { -cluster <cluster name> {-busDataSourceJndi <JNDI name>} | { -node <node name> -server <server name> } [-busName <bus name>] [-busDest <bus destination>] [-useBusFileStore [-fileStoreBaseDir <file store base dir>]] [debug]
# This script take the following parameters:
#     script_path:  path to this script
#     -cluster: cluster name (example: cluster1)
#     -busDataSourceJndi jndi name of existing datasource for SIBus (example: jdbc/xdms)
#     -node:  node name (example: myNode01)
#     -server:  server name (example: myServer1)
#     -busName: name of bus to create.  Optional, default is XDMSBus
#     -busDest: name of bus destination (topic space) to create.  Optional, default is SipNotifyTopicSpace
#     -useBusFileStore: use file stores instead of data stores in the service integration bus members
#     -fileStoreBaseDir: override the default filestore location.  Only used if -useBusFileStore is also set.
#           The following directories will be used:
#            log: <file store base directory>/<node name>.<server name>-<bus name>/log
#            permanent and temporary store:  <file store base directory>/<node name>.<server name>-<bus name>/store
#     debug:  optional, just pass the word 'debug' to enable debug
#  
#
# EITHER -cluster and -logDir and -permStoreDir and -tempStoreDir
#     OR -node and -server must be specified
#
#   Example for cluster: wsadmin -lang jython -f createXDMSMessaging.py -cluster cluster1 -busDataSourceJndi jdbc/sibDS debug 
#   Example for single server:  wsadmin -lang jython -f createXDMSMessaging.py -node myNode01 -server myServer1 debug 
#

import sys
import string
import os.path
from whrandom import choice

#############################################################################
# Generate a unique schema name for a bus member datasource.
#
# The default is to have the form:  XDMS_nodename_servername.  If this string is 
# over 30 characters long, just return the last 30 characters
#
# This function can be modified to return a different format
#############################################################################
def generateSchemaName(busName, clusterName, nodeName, serverName):
	schemaName = "XDMS" + "_" + nodeName + "_" + serverName
	if (len(schemaName) > 30):
		rand = randomStr()
		schemaName = schemaName[0:25]+"_"+rand
	return schemaName
#endDef

#############################################################################
# Generate a random sequence of characters
#
# Supply a positive integer parameter to receieve a string of a given
# length.  Otherwise, the function will return a string of length 4.
#
#############################################################################
def randomStr(length=4):
	if(length < 1):
		length=4;
	return ''.join([choice(string.letters) for i in range(length)])
#endDef


# Takes in a list from a WSAdmin object and creates a Jython list out of it
def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

# Strips out the []'s and the specified string
def stripOutStringAndBrackets(string, value):
	retVal = string.replace('[' + value, '')
	retVal = retVal.replace(']', '')
	retVal = retVal.strip()
	return retVal
#endDef


# Returns true (1) if the given cluster name is located
def isValidCluster( clusterParm ):
    global scope
    scope = AdminConfig.getid("/ServerCluster:"+clusterName+"/")
    if len(scope) != 0:
        return 1
    else:
        return 0
        
#endDef

# Get a list of members of a cluster
def getClusterMembers(clusterName):
	# get the cluster id
	cluster = AdminConfig.getid("/ServerCluster:" + clusterName + "/")

	membersString = AdminConfig.showAttribute(cluster, "members")
	members = wsadminToList(membersString)
	return members
#endDef


# Returns true (1) if the given node name is located
def isValidNode( nodeParm ):

#    global debug
    global scope
    valid = 0
    elementsString = AdminConfig.list('Cell')

    if debug:
        print 'elementsString= ' + elementsString + '\n'
    #endIf
        
    elements = wsadminToList(elementsString)
    for element in elements:  

        if debug:
            print 'element= ' + element + '\n'
        #endIf
        
        nodesString = AdminConfig.list('Node')
        
        if debug:
            print 'nodesString= ' + nodesString + '\n'
        #endIf
        
        nodes = wsadminToList(nodesString)
     
        for node in nodes:  
            nodeName = AdminConfig.showAttribute(node, 'name')
            
            if debug:
                print 'node ' + node + ' has name attribute = ' + nodeName + '\n'
            #endIf
             
            if (( nodeName == nodeParm ) ):
                valid = 1
                scope = node
                break 
            #endIf 
        #endFor 
    #endFor
    return valid
 #endDef 

# Returns true (1) if the given server name is located
def isValidServer( serverName ):

#    global debug
    valid = 0
    elementsString = AdminConfig.list('Cell')

    if debug:
        print 'elementsString= ' + elementsString + '\n'
    #endIf
        
    elements = wsadminToList(elementsString)
    for element in elements:  

        if debug:
            print 'element= ' + element + '\n'
        #endIf
        
        serversString = AdminConfig.list('Server')
        
        if debug:
            print 'serversString= ' + serversString + '\n'
        #endIf
        
        servers = wsadminToList(serversString)
     
        for server in servers:  
            srvName = AdminConfig.showAttribute(server, 'name')
            
            if debug:
                print 'server ' + server + ' has name attribute = ' + srvName + '\n'
            #endIf
             
            if (( srvName == serverName) ):
                valid = 1
                break 
            #endIf 
        #endFor 
    #endFor
    return valid
 #endDef 


# Prints the usage
def printUsage():
	print 'Usage:\n\
	wsadmin.sh -lang jython -f <script name> -node <node name> -server <server name> [debug]\n'
#endDef

# Prints the Copyright
def printCopyright():
	print '\n***************************************************************\n\n\
	Licensed Materials - Property of IBM\n\
	5724-O05\n\
	(c) Copyright IBM Corp. 2007 All Rights Reserved.\n\
	* Trademark of International Business Machines\n'
	print '***************************************************************\n'
#endDef

def createBus(busName, busDesc):

    AdminTask.createSIBus(["-bus", busName, "-description", busDesc, "-secure", "false"] )
     
#endDef 


# Create a filestore bus member with a filestore
def addBusMemberServerFilestore(busName, nodeName, serverName ):
    
	if (len(fileStoreBaseDir) > 0):
		uniqueDir = nodeName + "." + serverName + "-" + busName
		logDir = os.path.join(fileStoreBaseDir, uniqueDir, "log")
		storeDir = os.path.join(fileStoreBaseDir, uniqueDir, "store")

		if debug:
			print "  using directories: " + logDir + " (log), " + storeDir + " (permanent and temporary store)"
		#endIf
		
		AdminTask.addSIBusMember(["-bus", busName, "-node", nodeName, "-server", serverName, "-fileStore", \
		                          "-logDirectory", logDir, "-permanentStoreDirectory", storeDir, "-temporaryStoreDirectory", storeDir ] )
	else:
		if debug:
			print "  using default directories for log, permanent store, and temporary store"
		#endIf
	
		AdminTask.addSIBusMember(["-bus", busName, "-node", nodeName, "-server", serverName, "-fileStore"] )
	#endIf
#endDef 

def addBusMemberServerDatastore(busName, nodeName, serverName, dataSourceJndi, schemaName ):
    
    AdminTask.addSIBusMember(["-bus", busName, "-node", nodeName, "-server", serverName, "-dataStore", "-datasourceJndiName", dataSourceJndi, "-createDefaultDatasource", "FALSE", "-schemaName", schemaName ] )
    
#endDef 

# Add all the servers in a cluster as a server bus member
def addClusterServers(busName, clusterName, dataSourceJndi ):
	if debug:
		print "Adding servers in cluster " + clusterName + " to bus " + busName
	# get list of members
	members = getClusterMembers(clusterName)
	for member in members:
		nodeName = AdminConfig.showAttribute(member, "nodeName")
		serverName = AdminConfig.showAttribute(member, "memberName")
		if debug:
			print "Adding bus member for nodeName: " + nodeName + ", serverName: " + serverName
		try:
			if useBusFileStore:
				if debug:
					print "addBusMemberServerFilestore(" + busName + ", " + nodeName + ", " + serverName + ")"			
				addBusMemberServerFilestore(busName, nodeName, serverName)
			else:
				schemaName = generateSchemaName( busName, clusterName, nodeName, serverName)
				if debug:
					print "addBusMemberServerDatastore(" + busName + ", " + nodeName + ", " + serverName + ", " + dataSourceJndi + ", " + schemaName + ")"
				addBusMemberServerDatastore(busName, nodeName, serverName, dataSourceJndi, schemaName )
		except:
			temp = str(sys.exc_value)
			index = temp.find('already exists')
			if ( index < 0 ):
				print '    uncaught exception', sys.exc_type, sys.exc_value
			else:
				print '    bus member for ' + nodeName + ':' + serverName + ' already exists\n'
			#endIf
	#endFor
#endDef


def addBusMemberCluster(busName, clusterName, busDataSourceJndi, schemaName ):
    print 'schemaName = ' + schemaName + '.\n'
    
    if (len(schemaName)>0):
        if debug:
            print 'AdminTask.addSIBusMember(-bus, ' + busName + ' -cluster, ' + clusterName + ', -dataStore, -datasourceJndiName, ' + busDataSourceJndi + ', -schemaName, ' + schemaName + ')\n'
        #endIf

        AdminTask.addSIBusMember(["-bus", busName, "-cluster", clusterName, "-dataStore", "-datasourceJndiName", busDataSourceJndi, "-schemaName", schemaName ] )
    else:
        if debug:
            print 'AdminTask.addSIBusMember(-bus, ' + busName + ' -cluster, ' + clusterName + ', -dataStore, -datasourceJndiName, ' + busDataSourceJndi + ')\n'
        #endIf
        AdminTask.addSIBusMember(["-bus", busName, "-cluster", clusterName, "-dataStore", "-datasourceJndiName", busDataSourceJndi] )
    
#endDef 

def addBusDest(busName, destName ):
    
    AdminTask.createSIBDestination(["-bus", busName, "-name", destName, "-type", "TopicSpace"])
    
#endDef 


def createJMSCF(scope, busName, JMSCF_Name, JMSCF_jndiName):

# XDMS requires a ConnectionFactory 
# type parameter - optional 
#  value queue will cause a QueueConnectionFactory to be created
#  value topic will cause a TopicConnectionFactory to be created
#  Not Specified -  will cause a generic ConnectionFactory to be created

    AdminTask.createSIBJMSConnectionFactory(scope,\
       ["-name", JMSCF_Name, "-jndiName", JMSCF_jndiName, "-busName", busName] )
     
#endDef 

# busName
# name - name of topic (
# topicString - hierarchy (company/dept/mytopic)
# topicSpace - name of topic space

def createJMSTopic(scope, busName, name, jndiName, topicString, topicSpace):

    AdminTask.createSIBJMSTopic(scope,\
       ["-name", name, "-jndiName", jndiName, "-busName", busName, "-topicName", topicString, "-topicSpace", topicSpace] )
     
#endDef 

def createJMSQ(scope, busName, destName, name, jndiName, topicSpace):

    AdminTask.createSIBJMSTopic(scope,\
       ["-name", name, "-jndiName", jndiName, "-busName", busName, "-topicName", destName, "-topicSpace", topicSpace] )
     
#endDef 

def createJMSAS(scope, busName, destJndiName, JMSAS_Name, JMSAS_jndiName):

    AdminTask.createSIBJMSActivationSpec(scope,\
       ["-name", JMSAS_Name, "-jndiName", JMSAS_jndiName, "-busName", busName,\
        "-destinationJndiName", destJndiName, "-destinationType", "Topic"] )
     
#endDef 



#-----------------------------------------------------------------
# Main
#-----------------------------------------------------------------

busName = "XDMSBus"
busDesc = "XDMS Service Integration Bus"
busDataSourceJndi = "jdbc/xdmssib"
# destName = "XDMSDestination"
JMSCF_Name = "xdmsTCF"
JMSCF_jndiName = "jms/xdmsTCF"
# JMSQ_Name = "xdmsPublishT"
JMSAS_Name = "XDMSActivationSpec"
JMSAS_jndiName = "eis/XDMSActivationSpec"

SipNotifyTopicName = "xdmsPublishT"
SipNotifyTopicString = "XDMS/SIPNotify"
SipNotifyTopicSpace = "SipNotifyTopicSpace"
SipNotifyJndiName = "jms/xdmsPublishT"

nodeName = ""
scope = ""
scopeName = ""
serverName = ""
clusterName = ""
fileStore = ""
logDir = ""
permStoreDir = ""
tempStoreDir = ""
debug = 0
clusterInstall = 0
useBusFileStore = 0
fileStoreBaseDir = ""
printCopyright()

count = 0
arglen = len(sys.argv)
#print 'number of args received= ' + str(arglen) + '\n'

while (count < arglen):
        paramflag = sys.argv[count]
        count = count+1 

        if (cmp(paramflag, "-node") == 0):
                param = sys.argv[count]
                nodeName = param
        elif (cmp(paramflag, "-server") == 0):
                param = sys.argv[count]
                serverName = param
        elif (cmp(paramflag, "-busName") == 0):
                param = sys.argv[count]
                busName = param
        elif (cmp(paramflag, "-busDest") == 0):
                param = sys.argv[count]
                SipNotifyTopicSpace = param
        elif (cmp(paramflag, "-cluster") == 0):
                param = sys.argv[count]
                clusterName = param
                clusterInstall = 1
        elif (cmp(paramflag, "-busDataSourceJndi") == 0):
                param = sys.argv[count]
                busDataSourceJndi = param
        elif (cmp(paramflag, "-useBusFileStore") ==0):
                count = count-1
                useBusFileStore = 1
        elif (cmp(paramflag, "-fileStoreBaseDir") == 0):
                param = sys.argv[count]
                fileStoreBaseDir = param
        elif (cmp(paramflag, "debug") == 0):
                count = count-1
                debug = 1
        #endIf 

        count = count+1
#endWhile 


if ( not ( clusterInstall ) ):
    if ( (nodeName == "") or (serverName == "") ):
        print '-node and -server are required arguments\n'
        printUsage()
        sys.exit(0)
    #endIf
#endIf


if ( clusterInstall ):
    print 'XDMS messaging configured for cluster ' + clusterName
    SipNotifyTopicSpace = clusterName
else:
    print 'XDMS messaging configured for server ' + serverName + ' on node ' + nodeName
#endIf



if ( clusterInstall ):
    scopeName = "cluster " + clusterName
    if ( not ( isValidCluster(clusterName))):
        print 'Cluster ' + clusterName + ' specified is not valid '
        sys.exit(0)
else:
    scopeName = "node "+ nodeName
    if ( not ( isValidNode(nodeName))):
        print 'The specified Node (' + nodeName + ') is not configured on this cell'
        sys.exit(0)
    #endIf
    if ( not (isValidServer( serverName ))): 
        print 'The specified Server (' + serverName + ') is not configured on this node'
        sys.exit(0)
    #endIf
#endIf

if debug:
	print 'debug is enabled!\
	node:  ' + nodeName + '\n\
	server:  ' + serverName + '\n\
      scope:  ' + scope + '\n'
#endIf

print 'Adding SIBus ' + busName + ' .......\n'
try:
    createBus(busName, busDesc)
    print '    ' + busDesc + ' added\n'
except:
    temp = str(sys.exc_value)
    index = temp.find('already exists')
    if ( index < 0 ):
        print '    uncaught exception', sys.exc_type, sys.exc_value
    else:
        print '    SIBus ' + busName + ' already exists\n'
    #endIf

if ( clusterInstall ):
    print 'Adding servers in Cluster ' + clusterName + ' as bus members of SIBus ' +\
          busName + ' .......\n'
    try:
		addClusterServers(busName, clusterName, busDataSourceJndi)
		print 'Servers in cluster ' + clusterName + ' added as bus members of SIBus ' +\
              busName + '\n'
    except:
        temp = str(sys.exc_value)
        index = temp.find('already exists')
        if ( index < 0 ):
            print 'uncaught exception', sys.exc_type, sys.exc_value
        else:
            print clusterName + ' already added as a cluster member of SIBus ' +\
                  busName + '\n'
        #endIf

else:
    print 'Adding ' + nodeName + ':' + serverName + ' as a server member of SIBus ' +\
          busName + ' .......\n'
    try:
        addBusMemberServer(busName, nodeName, serverName)
        print '    ' + nodeName + ':' + serverName + ' added as a server member of SIBus ' +\
              busName + '\n'
    except:
        temp = str(sys.exc_value)
        index = temp.find('already exists')
        if ( index < 0 ):
            print '    uncaught exception', sys.exc_type, sys.exc_value
        else:
            print '    ' + nodeName + ':' + serverName + ' already a server member of bus ' +\
                  busName + '\n'
        #endIf
#endIf

print 'Adding Destination ' + SipNotifyTopicSpace + ' to SIBus ' +\
    busName + ' .......\n'
    
try:
    addBusDest(busName, SipNotifyTopicSpace)
    print '  Destination ' + SipNotifyTopicSpace + ' added to SIBus ' +\
          busName + '\n'
except:
    temp = str(sys.exc_value)
    index = temp.find('already exists')
    if ( index < 0 ):
        print 'uncaught exception', sys.exc_type, sys.exc_value
    else:
        print '  Destination ' + SipNotifyTopicSpace + ' already added to SIBus ' +\
              busName + '\n'
    #endIf


print 'Adding JMS Connection Factory ' + JMSCF_Name + ' to Node' +\
      nodeName + ' .......\n'

try:
    createJMSCF(scope, busName, JMSCF_Name, JMSCF_jndiName)

    print '    JMS Connection Factory ' + JMSCF_Name + ' added to ' +\
          scopeName + '\n'
except:
    temp = str(sys.exc_value)
    index = temp.find('existing J2CConnectionFactory')
    if ( index < 0 ):
        print '    uncaught exception', sys.exc_type, sys.exc_value
    else:
        print '    JMS Connection Factory ' + JMSCF_Name + ' already exists on ' +\
              scopeName + '\n'
    #endIf

print 'Adding JMS Topic ' + SipNotifyTopicName + ' to ' +\
      scopeName + ' .......\n'

try:

     createJMSTopic(scope, busName, SipNotifyTopicName, SipNotifyJndiName, SipNotifyTopicString, SipNotifyTopicSpace)
                   
except:
    temp = str(sys.exc_value)
    index = temp.find('existing J2CAdminObject')
    if ( index < 0 ):
        print '    uncaught exception', sys.exc_type, sys.exc_value
    else:
        print '    JMS Topic ' + SipNotifyTopicName + ' already exists on ' +\
              scopeName + '\n'
    #endIf

print 'Adding JMS Activation Spec ' + JMSAS_Name + ' to ' +\
      scopeName + ' .......\n'

try:
    createJMSAS(scope, busName, SipNotifyJndiName, JMSAS_Name, JMSAS_jndiName)
    print '    JMS Activation Spec ' + JMSAS_Name + ' added to ' +\
          scopeName + '\n'
except:
    temp = str(sys.exc_value)
    index = temp.find('existing J2CActivationSpec')
    if ( index < 0 ):
        print '    uncaught exception', sys.exc_type, sys.exc_value
    else:
        print '    JMS Activation Spec ' + JMSAS_Name + ' already exists on ' +\
              scopeName + '\n'
    #endIf



#prompt whether to save these changes
saveIt = raw_input("Save configuration changes? (y/n)")
if ( (saveIt == "y") or (saveIt == "Y")):
	# Save it! 
	AdminConfig.save()
	if debug:
		print 'Config has been saved'
	#endIf
	if ( clusterInstall ):
		print 'XDMS messaging configured for cluster ' + clusterName
	else:
		print 'XDMS messaging configured for server ' + serverName + ' on node ' + nodeName
	#endIf
else:
	AdminConfig.reset()
	if debug:
		print 'Config has not been saved'
	#endIf
#endIf
