#
# XDMS WSAdmin Jython script Uninstall Script
#
# This script deletes the service integration bus and the JMS artifacts used by the 
# IMS Service Platform XDMS component.
#
# Usage:
# wsadmin.sh -lang jython -f <script name> [-busName <bus name>] {  -cluster <cluster name> | -node <node name> -server <server name> } [-schemasFile <file name>] [-deleteBus ] [debug]
# This script takes the following parameters:
#     script name:  pathname of this script
#     -busName:  name of the bus that the cluster is using Optional, default is XDMSBus
#     -cluster: cluster name (example: cluster1)
#     -node:  node name (ie, myNode01), required if cluster is not specified
#     -server:  server name (ie, server1), required if cluster is not specified
#     -schemasFile: name of file to create that will list the schemas used by service integration bus members.  
#             Optional, default is <cluster name>schemas.lst.  You can use this list to delete the schema from the database
#     -deleteBus: Delete the entire bus, optional.  If this parameter is set, then the service 
#             integration bus is deleted.  Only use this parameter if you wish to delete 
#             _all_ artifacts of the bus. i.e. every cluster will have its bus objects deleted. 
#     debug:  optional, just pass the word 'debug' to enable debug
#
#   Example for cluster: wsadmin -lang jython -f deleteXdmsNotifyMessaging.py -cluster cluster1
#   Example for single server: wsadmin -lang jython -f-node mynode -server myserver1 debug 
#
#

import sys
import string
import os.path

# Takes in a list from a WSAdmin object and creates a Jython list out of it
def wsadminToList(inStr):
        outList=[]
        if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
                tmpList = inStr[1:-1].split(" ")
        else:
                tmpList = inStr.split("\n")  #splits for Windows or Linux
        for item in tmpList:
                item = item.rstrip();        #removes any Windows "\r"
                if (len(item)>0):
                        outList.append(item)
        return outList
#endDef

# Strips out the []'s and the specified string
def stripOutStringAndBrackets(string, value):
	retVal = string.replace('[' + value, '')
	retVal = retVal.replace(']', '')
	retVal = retVal.strip()
	return retVal
#endDef


# Returns true (1) if the given cluster name is located
def isValidCluster( clusterParm ):
    global scope
    scope = AdminConfig.getid("/ServerCluster:"+clusterName+"/")
    if len(scope) != 0:
        return 1
    else:
        return 0
        
#endDef

# Get a list of members of a cluster
def getClusterMembers(clusterName):
	# get the cluster id
	cluster = AdminConfig.getid("/ServerCluster:" + clusterName + "/")

	membersString = AdminConfig.showAttribute(cluster, "members")
	members = wsadminToList(membersString)
	return members
#endDef


# get a list of schema used by the server bus members of a cluster
def getClusterBusSchema(busName, clusterName):
	# list of schemas to return
	schemas=[]
	# get list of members
	# members = getClusterMembers(clusterName)
	cluster = AdminConfig.getid("/ServerCluster:" + clusterName + "/")
	members = getClusterMembers(clusterName)
	
	# iterate through members and get its engines
	for member in members:
		nodeName = AdminConfig.showAttribute(member, "nodeName")
		serverName = AdminConfig.showAttribute(member, "memberName")
		if debug:
			print "Getting schema for nodeName: " + nodeName + ", serverName: " + serverName
		
		# get SIB engines
		enginesString = AdminTask.listSIBEngines(["-bus", busName, "-node", nodeName, "-server", serverName])
		engines = wsadminToList(enginesString)
		
		# iterate through engines and get schema
		for engine in engines:
		
			messageStoreType = AdminConfig.showAttribute(engine, "messageStoreType")
			if debug:
				print "message store type = " + messageStoreType
			#endIf
			
			if (messageStoreType == "DATASTORE"):
				dataStore = AdminConfig.showAttribute(engine, "dataStore")
				schemaName = AdminConfig.showAttribute(dataStore, "schemaName")
				schemas.append(schemaName)
			#endIf
		#endFor
	#endFor
	return schemas
#endDef

# Create a file 
def writeDB2UninstallScript(schemas, clusterName):

	if len(schemas) > 0:
		#prompt for overwriting of existing file
		if (os.path.isfile(schemasFile)):
			overwrite = raw_input("File " + schemasFile + " already exists.  Overwrite? (y/n)")
			print overwrite
			if ( (overwrite != "y") and (overwrite != "Y")):
				print "Skipping schemas file creation"
				return
			#endIf
		#endIf
		
		f = open(schemasFile, "w")
		for schema in schemas:
			f.write(schema.upper() + "\n")
		#endFor
		f.close()
	else:
		print "No data stores, skipping schema file creation"
	#endIf
#endDef

# Returns true (1) if the given node name is located
def isValidNode( nodeParm ):

#    global debug
    global scope
    valid = 0
    elementsString = AdminConfig.list('Cell')

    if debug:
        print 'elementsString= ' + elementsString + '\n'
    #endIf
        
    elements = wsadminToList(elementsString)
    for element in elements:  

        if debug:
            print 'element= ' + element + '\n'
        #endIf
        
        nodesString = AdminConfig.list('Node')
        
        if debug:
            print 'nodesString= ' + nodesString + '\n'
        #endIf
        
        nodes = wsadminToList(nodesString)
     
        for node in nodes:  
            nodeName = AdminConfig.showAttribute(node, 'name')
            
            if debug:
                print 'node ' + node + ' has name attribute = ' + nodeName + '\n'
            #endIf
             
            if (( nodeName == nodeParm ) ):
                valid = 1
                scope = node
                break 
            #endIf 
        #endFor 
    #endFor
    return valid
 #endDef 

# Returns true (1) if the given server name is located
def isValidServer( serverName ):

#    global debug
    valid = 0
    elementsString = AdminConfig.list('Cell')

    if debug:
        print 'elementsString= ' + elementsString + '\n'
    #endIf
        
    elements = wsadminToList(elementsString)
    for element in elements:  

        if debug:
            print 'element= ' + element + '\n'
        #endIf
        
        serversString = AdminConfig.list('Server')
        
        if debug:
            print 'serversString= ' + serversString + '\n'
        #endIf
        
        servers = wsadminToList(serversString)
     
        for server in servers:  
            srvName = AdminConfig.showAttribute(server, 'name')
            
            if debug:
                print 'server ' + server + ' has name attribute = ' + srvName + '\n'
            #endIf
             
            if (( srvName == serverName) ):
                valid = 1
                break 
            #endIf 
        #endFor 
    #endFor
    return valid
 #endDef 


# Prints the usage
def printUsage():
	print 'Usage:\n\
	wsadmin.sh -lang jython -f <script name> -node <node name> -server <server name> -deleteBus <bus name> [debug]\n'
#endDef

# Prints the Copyright
def printCopyright():
	print '***************************************************************\n\n\
	Licensed Materials - Property of IBM\n\
	5724-O05\n\
	(c) Copyright IBM Corp. 2007 All Rights Reserved.\n\
	* Trademark of International Business Machines\n'
	print '***************************************************************'
#endDef

def deleteEntireBus(busName):

    AdminTask.deleteSIBus(["-bus", busName] )
     
#endDef 

def removeBusMemberServer(busName, nodeName, serverName ):
    
    AdminTask.removeSIBusMember(["-bus", busName, "-node", nodeName, "-server", serverName] )
    
#endDef 

def removeBusMemberCluster(busName, clusterName ):
 
    AdminTask.removeSIBusMember(["-bus", busName, "-cluster", clusterName] )
    
#endDef 

def deleteBusDestination(busName, destName ):
    
    AdminTask.deleteSIBDestination(["-bus", busName, "-name", destName] )
    
#endDef 

def deleteJMSCF(scope, busName, JMSCF_Name, JMSCF_jndiName):

    found = 0
        
    # PXNotify currently requires a QueueConnectionFactory 
    # type parameter - optional 
    #  value queue will list all QueueConnectionFactories
    #  value topic will list all TopicConnectionFactories
    #  Not Specified -  will list all generic ConnectionFactories

    cfsString = AdminTask.listSIBJMSConnectionFactories(scope, [] )

# listSIBJMSConnectionFactories is return a list of J2CConnectionFactory objects instead
# of a list of SIBJMSConnectionFactory objects and therefore the 'busName' attribute is
# not available
        
    if debug:
        print 'cfsString= ' + cfsString + '\n'
    #endIf
        
    cfs = wsadminToList(cfsString)
     
    for cf in cfs:  
        cfName = AdminConfig.showAttribute(cf, 'name')
            
        if debug:
            print 'cf ' + cf + ' has name attribute = ' + cfName + '\n'
        #endIf
             
        if (( cfName == JMSCF_Name) ):

            cfJndiName = AdminConfig.showAttribute(cf, 'jndiName')

            if debug:
                print 'cf ' + cf + ' has jndiName attribute = ' + cfJndiName + '\n'
            #endIf

            if (( cfJndiName == JMSCF_jndiName) ):

                # cfBusName = AdminConfig.showAttribute(cf, 'busName')
                
                # if debug:
                #    print 'cf ' + cf + ' has busName attribute = ' + cfBusName + '\n'
                #endIf

                # if (( cfBusName == busName) ):
                found = 1
                print '  JMS Connection Factory ' + JMSCF_Name + ' with JNDI Name ' +\
                          JMSCF_jndiName + ' on bus ' + busName + ' deleted\n'
                AdminTask.deleteSIBJMSConnectionFactory( cf )
                break
                # endIf
            #endIf 
        #endIf 
    #endFor 

    if ( not ( found ) ):
        print '  JMS Connection Factory ' + JMSCF_Name + ' with JNDI Name ' +\
              JMSCF_jndiName + ' on bus ' + busName + ' does not exists\n'
    #endIf 
   
#endDef 

def deleteJMSAS(scope, busName, JMSQ_jndiName, JMSAS_Name, JMSAS_jndiName):

    found = 0
        
    actvSpecsString = AdminTask.listSIBJMSActivationSpecs(scope)

# listSIBJMSConnectionFactories is return a list of J2CConnectionFactory objects instead
# of a list of SIBJMSConnectionFactory objects and therefore the 'busName' attribute is
# not available
        
    if debug:
        print 'actvSpecsString= ' + actvSpecsString + '\n'
    #endIf
        
    actvSpecs = wsadminToList(actvSpecsString)
     
    for actvSpec in actvSpecs:  
        asName = AdminConfig.showAttribute(actvSpec, 'name')
            
        if debug:
            print 'actvSpec ' + actvSpec + ' has name attribute = ' + asName + '\n'
        #endIf
             
        if (( asName == JMSAS_Name) ):

            asJndiName = AdminConfig.showAttribute(actvSpec, 'jndiName')

            if debug:
                print 'actvSpec ' + actvSpec + ' has jndiName attribute = ' + asJndiName + '\n'
            #endIf

            if (( asJndiName == JMSAS_jndiName) ):

                asDestJndiName = AdminConfig.showAttribute(actvSpec, 'destinationJndiName')
                
                if debug:
                    print 'actvSpec ' + actvSpec + ' has destinationJndiName attribute = ' + asDestJndiName + '\n'
                #endIf

                if (( asDestJndiName == JMSQ_jndiName) ):
                    found = 1
                    print '  JMS Activation Spec ' + JMSAS_Name + ' with JNDI Name ' +\
                          JMSAS_jndiName + ' on bus ' + busName + ' deleted\n'
                    AdminTask.deleteSIBJMSActivationSpec( actvSpec )
                    break
                #endIf
            #endIf 
        #endIf 
    #endFor 

    if ( not ( found ) ):
        print '  JMS Activation Spec' + JMSAS_Name + ' with JNDI Name ' +\
              JMSAS_jndiName + ' on bus ' + busName + ' does not exists\n'
    #endIf 
   
#endDef 



def deleteJMSTopic(scope, busName, destName, JMSTopic_Name, JMSTopic_jndiName):

    found = 0
        
    topicsString = AdminTask.listSIBJMSTopics(scope)

# listSIBJMSTopics returns a list of J2CAdminObject objects instead
# of a list of SIBJMSQueue objects and therefore the 'topicName' attribute is
# not available

    if debug:
        print 'topicsString= ' + topicsString + '\n'
    #endIf
        
    topics = wsadminToList(topicsString)

    for topic in topics:  
        topicName = AdminConfig.showAttribute(topic, 'name')
            
        if debug:
            print 'topic ' + topic + ' has name attribute = ' + topicName + '\n'
        #endIf
             
        if (( topicName == JMSTopic_Name) ):

            topicJndiName = AdminConfig.showAttribute(topic, 'jndiName')

            if debug:
                print 'topic ' + topic + ' has jndiName attribute = ' + topicJndiName + '\n'
            #endIf

            if (( topicJndiName == JMSTopic_jndiName) ):

                # queueName = AdminConfig.showAttribute(queue, 'queueName')
                
                # if debug:
                #    print 'queue ' + queue + ' has queueName attribute = ' + queueName + '\n'
                # endIf

                # if (( queueName == destName) ):
                found = 1
                print '  JMS Topic ' + JMSTopic_Name + ' with JNDI Name ' +\
                          JMSTopic_jndiName + ' on bus ' + busName + ' deleted\n'
                AdminTask.deleteSIBJMSTopic( topic )
                break
                # endIf
            #endIf 
        #endIf 
    #endFor 

    if ( not ( found ) ):
        print '  JMS Topic ' + JMSTopic_Name + ' with JNDI Name ' +\
              JMSTopic_jndiName + ' on bus ' + busName + ' does not exists\n'
    #endIf 
   
#endDef 



# Delete all the server bus members that are in a cluster
def removeClusterServersBusMembers(busName, clusterName ):
	if debug:
		print "Removing servers in cluster " + clusterName + " from bus " + busName
	# get list of members
	members = getClusterMembers(clusterName)
	for member in members:
		nodeName = AdminConfig.showAttribute(member, "nodeName")
		serverName = AdminConfig.showAttribute(member, "memberName")
		if debug:
			print "Removing bus member for nodeName: " + nodeName + ", serverName: " + serverName
		try:
			if debug:
				print "removeBusMemberServer(" + busName + ", " + nodeName + ", " + serverName + ")"
			#endIf
			removeBusMemberServer(busName, nodeName, serverName )
		except:
			temp = str(sys.exc_value)
			index = temp.find('already exists')
			if ( index < 0 ):
				print '    uncaught exception', sys.exc_type, sys.exc_value
			else:
				print '    bus member for ' + nodeName + ':' + serverName + ' already exists\n'
			#endIf
	#endFor
#endDef



#-----------------------------------------------------------------
# Main
#-----------------------------------------------------------------

busName = "XDMSBus"
busDesc = "XDMS Service Integration Bus"
JMSCF_Name = "xdmsTCF"
JMSCF_jndiName = "jms/xdmsTCF"
JMSAS_Name = "XDMSActivationSpec"
JMSAS_jndiName = "eis/XDMSActivationSpec"

SipNotifyTopicName = "xdmsPublishT"
SipNotifyTopicString = "XDMS/SIPNotify"
SipNotifyTopicSpace = "SipNotifyTopicSpace"
SipNotifyJndiName = "jms/xdmsPublishT"

nodeName = ""
scope = ""
serverName = ""
clusterName = ""
schemasFile = ""
debug = 0
clusterInstall = 0
deleteBus = 0

printCopyright()

count = 0
arglen = len(sys.argv)
#print 'number of args received= ' + str(arglen) + '\n'

while (count < arglen):
        paramflag = sys.argv[count]
        count = count+1 

        if (cmp(paramflag, "-node") == 0):
                param = sys.argv[count]
                nodeName = param
        elif (cmp(paramflag, "-server") == 0):
                param = sys.argv[count]
                serverName = param
        elif (cmp(paramflag, "-overrideBusName") == 0):
                param = sys.argv[count]
                busName = param
        elif (cmp(paramflag, "-overrideBusDest") == 0):
                param = sys.argv[count]
                destName = param
        elif (cmp(paramflag, "-cluster") == 0):
                param = sys.argv[count]
                clusterName = param
                clusterInstall = 1
        elif (cmp(paramflag, "-deleteBus") == 0):
                count = count-1
                deleteBus = 1
        elif (cmp(paramflag, "-schemasFile") == 0):
                param = sys.argv[count]
                schemasFile = param
        elif (cmp(paramflag, "debug") == 0):
                count = count-1
                debug = 1
        #endIf 

        count = count+1
#endWhile 

if ( not ( clusterInstall ) ):
    if ( (nodeName == "") or (serverName == "") ):
        print '-node and -server are required arguments\n'
        printUsage()
        sys.exit(0)
    #endIf
#endIf


if ( clusterInstall ):
    SipNotifyTopicSpace = clusterName
    
    # default the schemasFile 
    if ( schemasFile == "" ):
        schemasFile = clusterName + "schemas.lst"
    #endIf

    if ( not ( isValidCluster(clusterName))):
        print 'Cluster ' + clusterName + ' specified is not valid '
        sys.exit(0)
    #endIf
    
    schemas = getClusterBusSchema(busName, clusterName)
    writeDB2UninstallScript(schemas, clusterName)

else:
    if ( not ( isValidNode(nodeName))):
        print 'The specified Node (' + nodeName + ') is not configured on this cell'
        sys.exit(0)
    #endIf
    if ( not (isValidServer( serverName ))): 
        print 'The specified Server (' + serverName + ') is not configured on this node'
        sys.exit(0)
    #endIf
#endIf

if debug:
	print 'debug is enabled!\
	node:  ' + nodeName + '\n\	server:  ' + serverName + '\n\
	scope:  ' + scope + '\n'
#endIf

print 'Deleting JMS Activation Spec ' + JMSAS_Name + ' .......\n'
deleteJMSAS(scope, busName, SipNotifyJndiName, JMSAS_Name, JMSAS_jndiName)

print 'Deleting JMS Topic ' + SipNotifyTopicName + ' .......\n'
# deleteJMSQ(scope, busName, destName, JMSQ_Name, JMSQ_jndiName)
deleteJMSTopic(scope, busName, SipNotifyTopicSpace, SipNotifyTopicName, SipNotifyJndiName)

print 'Deleting JMS Connection Factory ' + JMSCF_Name + ' .......\n'
deleteJMSCF(scope, busName, JMSCF_Name, JMSCF_jndiName)

# Deleting the SIBus will remove any configured members and destinations as well
if deleteBus:
    print 'Deleting SIBus ' + busName +  ' .......\n'
    try:
        deleteEntireBus(busName)
        print 'SIBus ' + busName + ' was deleted\n'
    except:
        temp = str(sys.exc_value)
        index = temp.find('not found')
        if ( index < 0 ):
            print 'uncaught exception', sys.exc_type, sys.exc_value
        else:
            print 'SIBus ' + busName + ' does not exist\n'
        #endIf
else:
	# Delete Bus members and bus destination
	print "Deleting bus destination " + SipNotifyTopicSpace + " from bus " + busName
	deleteBusDestination(busName, SipNotifyTopicSpace )
	 
	if (clusterInstall):
		# Remove the server bus members that are part of the cluster
		print "Removing server bus members that are part of cluster " + clusterName + " from bus " + busName
		removeClusterServersBusMembers(busName, clusterName )
	else:
		# Remove the server bus member
		print "Removing server bus member from bus " + busName
		removeClusterServersBusMembers(busName, clusterName )
		if debug:
			print "removeBusMemberServer(" + busName + ", " + nodeName + ", " + serverName + ")"
		#endIf
		removeBusMemberServer(busName, nodeName, serverName )
	#endIf
#endIf

#prompt whether to save these changes
saveIt = raw_input("Save configuration changes? (y/n)")
if ( (saveIt == "y") or (saveIt == "Y")):
	# Save it! 
	AdminConfig.save()
	if debug:
		print 'Config has been saved'
	#endIf
	if ( clusterInstall ):
		print 'XDMS messaging unconfigured for cluster ' + clusterName
	else:
		print 'XDMS messaging unconfigured for server ' + serverName + ' on node ' + nodeName
	#endIf
else:
	AdminConfig.reset()
	if debug:
		print 'Config has not been saved'
	#endIf
#endIf


